"""Configuration module for logger."""
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
# The background is set with 40 plus the number of the color, and the foreground with 30

# These are the sequences need to get coloured outputs
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ = "\033[1m"

COLOURS = {
    "WARNING": YELLOW,
    "INFO": GREEN,
    "DEBUG": BLUE,
    "CRITICAL": YELLOW,
    "ERROR": RED,
}



def clear_handlers(logger):
    """
    Remove all handlers from a logger.
    """
    handlers = getattr(logger, "handlers", [])
    for handler in handlers:
        logger.removeHandler(handler)

    return logger



"""Module to collect all tests for the logger module."""

import pytest
from src.coloured_logger.logger import ColouredLogger, ColouredFormatter, formatter_message
import logging


def test_formatter_message_with_color():
    formatted_message = formatter_message("$BOLDHello$RESET", use_color=True)
    assert formatted_message == "\033[1mHello\033[0m"

def test_formatter_message_without_color():
    formatted_message = formatter_message("$BOLDHello$RESET", use_color=False)
    assert formatted_message == "Hello"

def test_coloured_logger_creation(caplog):
    caplog.set_level(logging.INFO)
    logger = ColouredLogger("test_logger")

    # Ensure the logger has a stream handler
    assert any(isinstance(handler, logging.StreamHandler) for handler in logger.handlers)

    # Ensure the formatter is a ColouredFormatter instance
    formatter = logger.handlers[0].formatter
    assert isinstance(formatter, ColouredFormatter)

def test_coloured_formatter_format():
    record = logging.LogRecord('test_logger', logging.INFO, '/path/to/source.py', 42, 'Test message', (), None, 'format')
    formatter = ColouredFormatter(ColouredLogger.FORMAT, use_color=True)

    # Ensure the log message is formatted with colours
    formatted_message = formatter.format(record)
    
    expected_message = (
        "\033[1m[test_logger             ]\033[0m[\033[1;32mINFO             \033[0m]  Test message (\033[1m/path/to/source.py\033[0m:42)"
    )
    assert formatted_message[55:67] == expected_message[66:78]
    assert formatted_message.split("[")[3] == expected_message.split("[")[5].replace(" ", "")



"""Module that contains the definition of a customised logger class."""
# TODO: Move this into a separate package

import logging

from src.coloured_logger.logger_config import COLOURS, RESET_SEQ, COLOR_SEQ, BOLD_SEQ


def formatter_message(message, use_color=True):
    if use_color:
        message = message.replace("$RESET", RESET_SEQ).replace("$BOLD", BOLD_SEQ)
    else:
        message = message.replace("$RESET", "").replace("$BOLD", "")
    return message


class ColouredFormatter(logging.Formatter):
    def __init__(self, msg, use_color=True, colours=COLOURS):
        logging.Formatter.__init__(self, msg)
        self.use_color = use_color
        self.colours = colours

    def format(self, record):
        levelname = record.levelname
        if self.use_color and levelname in self.colours:
            levelname_color = (
                COLOR_SEQ % (30 + self.colours[levelname]) + levelname + RESET_SEQ
            )
            record.levelname = levelname_color
        return logging.Formatter.format(self, record)


# Custom logger class with multiple destinations (console, file, etc.)
class ColouredLogger(logging.Logger):
    """ColouredLogger Object to log messages with colours."""

    FORMAT = "[$BOLD%(name)-20s$RESET][%(levelname)-18s]  %(message)s ($BOLD%(filename)s$RESET:%(lineno)d)"

    def __init__(self, name: str, colours: dict = COLOURS):
        logging.Logger.__init__(self, name, logging.DEBUG)

        self.colours = colours
        self.colour_format = formatter_message(message=self.FORMAT, use_color=True)
        colour_formatter = ColouredFormatter(
            self.colour_format, use_color=True, colours=self.colours
        )

        console = logging.StreamHandler()
        console.setFormatter(colour_formatter)

        self.addHandler(console)
        return


class Logger(logging.Logger):
    FORMAT = "[$BOLD%(name)-20s$RESET][%(levelname)-18s]  %(message)s ($BOLD%(filename)s$RESET:%(lineno)d)"

    def __init__(self, name, colours=COLOURS, level="INFO"):
        super().__init__(name=name, level=level)
        self.logger = logging.getLogger(name)
        self.logger.setLevel(level)

        colour_format = formatter_message(message=self.FORMAT, use_color=True)
        colour_formatter = ColouredFormatter(
            colour_format, use_color=True, colours=colours
        )

        console = logging.StreamHandler()
        console.setFormatter(colour_formatter)

        self.logger.addHandler(console)

    def info(self, message, *args, **kwargs):
        self.logger.info(message, *args, **kwargs)

    def debug(self, message, *args, **kwargs):
        self.logger.debug(message, *args, **kwargs)

    def warning(self, message, *args, **kwargs):
        self.logger.warning(message, *args, **kwargs)

    def error(self, message, *args, **kwargs):
        self.logger.error(message, *args, **kwargs)

    def critical(self, message, *args, **kwargs):
        self.logger.critical(message, *args, **kwargs)



""" Setup file for the package."""
#!/usr/bin/env python3
import os
from src import coloured_logger
from src.coloured_logger import (
    __version__,
    __description__,
    __author__,
    __email__,
    __license__,
    __title__,
)
from setuptools import setup, find_packages

pypi_name = "coloured-logger"
pkg_name = "coloured_logger"


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


# Read requirements.txt
with open("requirements.txt") as f:
    requirements = f.read().splitlines()

s = setup(
    name=pypi_name,
    version=__version__,
    license=__license__,
    description=__description__,
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    keywords="security,scanner",
    url="https://github.com/oscar-defelice/%s" % pypi_name,
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    install_requires=requirements,
    python_requires=">= 3.9",
    author=__author__,
    author_email=__email__,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
    ],
)



__author__ = "Oscar de Felice"
__email__ = "oscar.defelice@gmail.com"
__title__ = "Coloured Logger"
__description__ = "A coloured logger for Python"
__version__ = "1.0.0"
__license__ = "MIT"

# Path: src/coloured_logger/__init__.py
from src.coloured_logger.logger import Logger

__all__ = [
    "logger_config",
    "logger",
]
